# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import sys
try:
    import qrencode
except ImportError:
    print("You need to install the python-qrencode package")
    sys.exit(1)

from PIL import Image
from PIL import ImageOps
import cairo
import array


class QRCodeSize(object):
    SMALL = 120
    MEDIUM = 230
    LARGE = 350


class QRCodeOutput(object):
    PIL_IMAGE = 0
    CAIRO_SURFACE = 1


class QRCode(object):

    def __init__(self, output_size=QRCodeSize.MEDIUM):
        self.output_size = output_size
        self.qrcode_size = 0
        self.pixel_size = 1
        self.image = None
        self.cr_surface = None

    def encode(self, text, current_color_fg=None, previous_color_fg=None,
               current_color_bg=None, previous_color_bg=None, border=True,
               output_type=QRCodeOutput.PIL_IMAGE):
        '''Encodes the given text and returns a QR code in the given format'''

        if output_type == QRCodeOutput.PIL_IMAGE:
            qr_code = self._encode_to_pil(text, current_color_fg,
                previous_color_fg, current_color_bg, previous_color_bg,
                border)
        elif output_type == QRCodeOutput.CAIRO_SURFACE:
            qr_code = self._encode_to_cairo(text, current_color_fg,
                previous_color_fg, current_color_bg, previous_color_bg,
                border)

        return qr_code

    def _encode_to_cairo(self, text, current_color_fg=None,
                         previous_color_fg=None, current_color_bg=None,
                         previous_color_bg=None, border=True):
        '''Encodes the given text and returns a Cairo surface'''
        self._encode_to_pil(text, current_color_fg, previous_color_fg,
                            current_color_bg, previous_color_bg, border)

        # Convert the Python PIL image to a Cairo surface.
        # Notice the conversion to BGRA that Cairo expects
        # See http://cairographics.org/pythoncairopil/ and
        # http://mail.gnome.org/archives/gtkmm-list/2007-May/msg00111.html
        bytearr = array.array('B', self.image.tobytes("raw", "BGRA", 0, 1))
        height, width = self.image.size
        #print self.image.size

        self.cr_surface = cairo.ImageSurface.create_for_data(bytearr,
                                                     cairo.FORMAT_ARGB32,
                                                     width, height,
                                                     width * 4)

        return self.cr_surface

    def _encode_to_pil(self, text, current_color_fg=None,
                         previous_color_fg=None, current_color_bg=None,
                         previous_color_bg=None, border=True):
        '''Encodes the given text and returns a Python PIL Image,
        changing colors if necessary'''

        # Returns an 8-bit pixel, black and white image (mode L)
        version, self.qrcode_size, self.image = qrencode.encode(text)

        if (current_color_fg != previous_color_fg) or \
           (current_color_bg != previous_color_bg):
            self.image = ImageOps.colorize(self.image,
                                           (current_color_fg[0],
                                           current_color_fg[1],
                                           current_color_fg[2]),
                                           (current_color_bg[0],
                                           current_color_bg[1],
                                           current_color_bg[2]))

        # Returns a 4x8-bit pixel, true colour with transparency mask image
        # (mode RGBA)
        self.image = self.image.convert('RGBA')
        
        self._resize()
        if border:
            self._add_border(current_color_bg)

        return self.image

    def _resize(self):
        self._set_pixel_size(self.qrcode_size)
        #print self.pixel_size, self.qrcode_size
        self.image = self.image.resize((self.pixel_size * self.qrcode_size,
                                        self.pixel_size * self.qrcode_size),
                                       Image.NEAREST)

    def _set_pixel_size(self, barcode_orig_size):
        pixel_size = 1

        #print pixel_size * barcode_orig_size, self.output_size
        while (pixel_size * barcode_orig_size < self.output_size):
            pixel_size += 1

        while ((pixel_size * barcode_orig_size > self.output_size) or \
                ((pixel_size * barcode_orig_size) % 2 != 0) or \
                (pixel_size * barcode_orig_size + 2 * pixel_size > \
                    self.output_size)):
            pixel_size -= 1

            # FIXME: it needs to automatically go up to the next size,
            # rather than hardcoding it as now
            if pixel_size == 0:
                self.output_size = QRCodeSize.MEDIUM

        self.pixel_size = pixel_size

    def _add_border(self, current_color_bg=None):
        '''Adds a border to the QR code'''
        if current_color_bg:
            fill = (int(current_color_bg[0]), int(current_color_bg[1]),
                    int(current_color_bg[2]), 255)
        else:
            fill = 'white'
        # Add a border
        border_size = int((self.output_size - self.image.size[0]) / 2)
        self.image = ImageOps.expand(self.image, border=border_size,
                                     fill=fill)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

### DO NOT EDIT THIS FILE ###

from gi.repository import Gio, Gtk # pylint: disable=E0611
import logging
logger = logging.getLogger('qreator_lib')

from . helpers import get_builder, show_uri, get_help_uri

# This class is meant to be subclassed by QreatorWindow.  It provides
# common functions and some boilerplate.
class Window(Gtk.Window):
    __gtype_name__ = "Window"

    # To construct a new instance of this method, the following notable 
    # methods are called in this order:
    # __new__(cls)
    # __init__(self)
    # finish_initializing(self, builder)
    # __init__(self)
    #
    # For this reason, it's recommended you leave __init__ empty and put
    # your initialization code in finish_initializing
    
    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated BaseQreatorWindow object.
        """
        builder = get_builder('QreatorWindow')
        new_object = builder.get_object("qreator_window")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called while initializing this instance in __new__

        finish_initializing should be called after parsing the UI definition
        and creating a QreatorWindow object with it in order to finish
        initializing the start of the new QreatorWindow instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self, True)
        self.PreferencesDialog = None # class
        self.preferences_dialog = None # instance
        self.AboutDialog = None # class

        self.settings = Gio.Settings("net.launchpad.qreator")
        self.settings.connect('changed', self.on_preferences_changed)

        # Optional application indicator support
        # Run 'quickly add indicator' to get started.
        # More information:
        #  http://owaislone.org/quickly-add-indicator/
        #  https://wiki.ubuntu.com/DesktopExperienceTeam/ApplicationIndicators
        try:
            from qreator import indicator
            # self is passed so methods of this class can be called from indicator.py
            # Comment this next line out to disable appindicator
            self.indicator = indicator.new_application_indicator(self)
        except ImportError:
            pass

    def on_mnu_contents_activate(self, widget, data=None):
        show_uri(self, "ghelp:%s" % get_help_uri())

    def on_mnu_about_activate(self, widget, data=None):
        """Display the about box for qreator."""
        if self.AboutDialog is not None:
            about = self.AboutDialog() # pylint: disable=E1102
            response = about.run()
            about.destroy()

    def on_mnu_preferences_activate(self, widget, data=None):
        """Display the preferences window for qreator."""

        """ From the PyGTK Reference manual
           Say for example the preferences dialog is currently open,
           and the user chooses Preferences from the menu a second time;
           use the present() method to move the already-open dialog
           where the user can see it."""
        if self.preferences_dialog is not None:
            logger.debug('show existing preferences_dialog')
            self.preferences_dialog.present()
        elif self.PreferencesDialog is not None:
            logger.debug('create new preferences_dialog')
            self.preferences_dialog = self.PreferencesDialog() # pylint: disable=E1102
            self.preferences_dialog.connect('destroy', self.on_preferences_dialog_destroyed)
            self.preferences_dialog.show()
        # destroy command moved into dialog to allow for a help button

    def on_mnu_close_activate(self, widget, data=None):
        """Signal handler for closing the QreatorWindow."""
        self.destroy()

    def on_destroy(self, widget, data=None):
        """Called when the QreatorWindow is closed."""
        # Clean up code for saving application state should be added here.
        Gtk.main_quit()

    def on_preferences_changed(self, settings, key, data=None):
        logger.debug('preference changed: %s = %s' % (key, str(settings.get_value(key))))

    def on_preferences_dialog_destroyed(self, widget, data=None):
        '''only affects gui
        
        logically there is no difference between the user closing,
        minimising or ignoring the preferences dialog'''
        logger.debug('on_preferences_dialog_destroyed')
        # to determine whether to create or present preferences_dialog
        self.preferences_dialog = None




# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

# VCARD reference http://tools.ietf.org/html/rfc6350

# TODO:
# PHOTO, BDAY, LABEL, MAILER, TZ, GEO, LOGO, AGENT,
# REV, SOUND, UID, VERSION, KEY
# X-ABUID, X-ANNIVERSARY, X-ASSISTANT, X-MANAGER
# X-SPOUSE, X-GENDER, X-AIM, X-ICQ, X-JABBER, X-MSN,
# X-YAHOO, X-TWITTER, X-SKYPE, X-SKYPE-USERNAME,
# X-GADUGADU, X-GROUPWISE, X-MS-IMADDRESS, X-MS-CARDPICTURE
# X-PHONETIC-FIRST-NAME, X-PHONETIC-LAST-NAME
# X-MOZILLA-HTML, X-MOZILLA-PROPERTY, X-EVOLUTION-ANNIVERSARY
# X-EVOLUTION-ASSISTANT ,X-EVOLUTION-BLOG-URL, X-EVOLUTION-FILE-AS
# X-EVOLUTION-MANAGER, X-EVOLUTION-SPOUSE, X-EVOLUTION-VIDEO-URL
# X-EVOLUTION-CALLBACK, X-EVOLUTION-RADIO, X-EVOLUTION-TELEX
# X-EVOLUTION-TTYTDD, X-KADDRESSBOOK-BlogFeed, X-KADDRESSBOOK-X-Anniversary
# X-KADDRESSBOOK-X-AssistantsName, X-KADDRESSBOOK-X-IMAddress,
# X-KADDRESSBOOK-X-ManagersName, X-KADDRESSBOOK-X-Office
# X-KADDRESSBOOK-X-Profession, X-KADDRESSBOOK-X-SpouseName

from .GtkHelpers import clear_text_entry, show_clear_icon
from gi.repository import Gtk, GObject
from qreator_lib.helpers import get_data_file
from qreator_lib.i18n import _
import vobject


ENTRY_TYPES = [(_("E-mail"), "email"),
               (_("Phone"), "tel"),
               (_("Address"), "adr"),
               (_("Website"), "url"),
               (_("Nickname"), "nickname"),
               (_("Title"), "title"),
               (_("Role"), "role"),
               (_("Note"), "note"),
               #(_("Organization"), "org"), # python-vobject/+bug/1010104
               ]

# TRANSLATORS:
# text: Indicates that the telephone number supports text messages (SMS).
# voice: Indicates a voice telephone number.
# fax: Indicates a facsimile telephone number.
# cell: Indicates a cellular or mobile telephone number.
# video: Indicates a video conferencing telephone number.
# pager: Indicates a paging device telephone number.
# textphone: Indicates a telecommunication device for people with
# hearing or speech difficulties.
PHONE_TYPES = [(_("Cell"), "cell"),
               (_("Voice"), "voice"),
               (_("Fax"), "fax"),
               (_("Video"), "video"),
               (_("Text"), "text"),
               (_("Pager"), "pager"),
               (_("Textphone"), "textphone"),
               ]

USAGE_TYPES = [(_("Home"), "home"),
               (_("Work"), "work")]


def on_icon(widget=None, icon=None, button=None):
    clear_text_entry(widget, icon)


class VCardEntry(object):
    """The class gets two variables:
    instances and row

    instances acts as a registry of elements in the vcard object
    and row will keep track of the latest position to add elements
    to the grid
    grid is a variable that is populated by the individual
    VCardEntry objects.
    """
    instances = []

    def __init__(self, entry_id, get_widget, widget_name):
        self.entry_id = entry_id
        self.entry_field = ENTRY_TYPES[entry_id][1]
        self.entry_text = ENTRY_TYPES[entry_id][0]
        self.grid = get_widget(widget_name)
        VCardEntry.instances.append(self)

    def add_sep(self, on_changed):
        """Attaches a separator and a removal button to the grid.
        The grid assumes that there are 3 columns:
        label, main stuff, removalbutton
        Returns the removal button and the new number of rows
        """
        sep = Gtk.Separator()
        sep.set_margin_top(3)
        sep.set_margin_bottom(3)
        sep.set_margin_right(6)
        sep.show()
        self.grid.attach(sep, 0, VCardEntry.row, 3, 1)

        image = Gtk.Image()
        image.set_from_icon_name(
                'edit-delete-symbolic', Gtk.IconSize.BUTTON)
        image.show()
        remove_button = Gtk.Button()
        remove_button.add(image)
        remove_button.set_margin_right(6)
        self.grid.attach(
            remove_button, 2, VCardEntry.row + 1, 1, 1)
        remove_button.set_tooltip_text(_('Remove this field'))
        remove_button.show()
        remove_button.connect("clicked", self.remove_me)
        remove_button.connect("clicked", on_changed)

        label = Gtk.Label(self.entry_text + ":")
        label.set_alignment(0, 0.5)
        #label.set_margin_top(3)
        label.set_margin_right(6)
        label.set_margin_left(3)
        label.show()
        self.grid.attach(
            label, 0, VCardEntry.row + 1, 1, 1)

        self.to_remove = [sep, label, remove_button]
        VCardEntry.row += 1

    def remove_me(self, widget):
        for i in self.to_remove:
            self.grid.remove(i)
        VCardEntry.instances.remove(self)

    def create_widget(self, on_changed):
        self.add_sep(on_changed)
        self.entry = Gtk.Entry()
        self.entry.set_hexpand(True)
        self.entry.set_margin_right(6)
        self.entry.set_margin_left(6)
        self.entry.connect("changed", on_changed)
        self.entry.connect("icon-press", on_icon)
        self.entry.show()
        self.grid.attach(
            self.entry, 1, VCardEntry.row, 1, 1)
        VCardEntry.row += 1
        self.to_remove.append(self.entry)

    def add_entries(self, card):
        text = self.entry.get_text()
        if text:
            card.add(self.entry_field).value = text
        return card


class GenericWithUsage(VCardEntry):
    """Adds a generic field with a label, a usage type,
    and  a entry field to the grid.
    """
    def create_widget(self, on_changed):
        self.add_sep(on_changed)
        self.box = Gtk.Box(False)
        self.box.set_orientation(Gtk.Orientation.HORIZONTAL)

        self.entry = Gtk.Entry()
        self.entry.set_hexpand(True)
        self.entry.set_margin_right(6)
        self.entry.set_margin_left(6)
        self.entry.connect("changed", on_changed)
        self.entry.connect("icon-press", on_icon)
        self.box.pack_end(self.entry, True, True, 0)

        self.usagecombo = Gtk.ComboBoxText()
        for t in USAGE_TYPES:
            self.usagecombo.append_text(t[0])
        self.usagecombo.set_active(0)
        self.usagecombo.connect("changed", on_changed)
        self.box.pack_end(self.usagecombo, False, False, 0)

        self.box.show_all()
        self.grid.attach(
            self.box, 1, VCardEntry.row, 1, 1)
        VCardEntry.row += 1
        self.to_remove.append(self.box)

    def add_entries(self, card):
        text = self.entry.get_text()
        if text:
            card.add(self.entry_field).value = text
            getattr(card,
                    self.entry_field).type_param = USAGE_TYPES[
                self.usagecombo.get_active()][1]
        return card


class Phone(VCardEntry):
    """Adds a phone field with a label, a usage type,
    a phone type and  a entry field to the grid.
    """
    def create_widget(self, on_changed):
        self.add_sep(on_changed)
        self.box = Gtk.Box(False)
        self.box.set_orientation(Gtk.Orientation.HORIZONTAL)

        self.entry = Gtk.Entry()
        self.entry.set_hexpand(True)
        self.entry.set_margin_right(6)
        self.entry.set_margin_left(6)
        self.entry.connect("changed", on_changed)
        self.entry.connect("icon-press", on_icon)
        self.box.pack_end(self.entry, True, True, 0)

        self.usagecombo = Gtk.ComboBoxText()
        for t in USAGE_TYPES:
            self.usagecombo.append_text(t[0])
        self.usagecombo.set_active(0)
        self.usagecombo.connect("changed", on_changed)
        self.box.pack_end(self.usagecombo, False, False, 0)

        self.phonecombo = Gtk.ComboBoxText()
        for t in PHONE_TYPES:
            self.phonecombo.append_text(t[0])
        self.phonecombo.set_active(0)
        self.phonecombo.connect("changed", on_changed)
        self.box.pack_end(self.phonecombo, False, False, 0)

        self.box.show_all()
        self.grid.attach(
            self.box, 1, VCardEntry.row, 1, 1)
        VCardEntry.row += 1
        self.to_remove.append(self.box)

    def add_entries(self, card):
        text = self.entry.get_text()
        if text:
            card.add(self.entry_field).value = text
            tp = [USAGE_TYPES[self.usagecombo.get_active()][1],
                  PHONE_TYPES[self.phonecombo.get_active()][1]]
            getattr(card,
                    self.entry_field).type_param = tp
        return card


class Address(VCardEntry):
    def create_widget(self, on_changed):
        self.add_sep(on_changed)

        self.street = Gtk.Entry()
        self.street.set_margin_right(6)
        self.street.set_margin_left(6)
        self.street.set_margin_top(3)
        self.street.set_margin_bottom(3)
        self.postbox = Gtk.Entry()
        self.postbox.set_margin_right(6)
        self.postbox.set_margin_left(6)
        self.postbox.set_margin_top(3)
        self.postbox.set_margin_bottom(3)
        self.city = Gtk.Entry()
        self.city.set_margin_right(6)
        self.city.set_margin_left(6)
        self.city.set_margin_top(3)
        self.city.set_margin_bottom(3)
        self.region = Gtk.Entry()
        self.region.set_margin_right(6)
        self.region.set_margin_left(6)
        self.region.set_margin_top(3)
        self.region.set_margin_bottom(3)
        self.code = Gtk.Entry()
        self.code.set_margin_right(6)
        self.code.set_margin_left(6)
        self.code.set_margin_top(3)
        self.code.set_margin_bottom(3)
        self.country = Gtk.Entry()
        self.country.set_margin_right(6)
        self.country.set_margin_left(6)
        self.country.set_margin_top(3)
        self.country.set_margin_bottom(3)

        grid = Gtk.Grid()
        grid.set_hexpand(True)
        for n, item  in enumerate([(self.street, _("Street")),
                                   (self.city, _("City")),
                                   (self.region, _("Region")),
                                   (self.code, _("Zip code")),
                                   (self.country, _("Country")),
                                   (self.postbox, _("Postbox"))]):
            e, t = item
            #e.set_placeholder_text('[{}]'.format(t))
            e.connect("changed", on_changed)
            e.connect("icon-press", on_icon)
            e.set_hexpand(True)
            label = Gtk.Label(t + ": ")
            label.set_alignment(0, 0.5)
            label.set_margin_right(6)
            label.set_margin_left(6)
            label.set_margin_top(3)
            label.set_margin_bottom(3)
            grid.attach(label, 1, n, 1, 1)
            grid.attach(e, 2, n, 1, 1)
            self.grid.attach(
                grid, 1, VCardEntry.row, 1, 1)
            self.to_remove.append(grid)

        self.usagecombo = Gtk.ComboBoxText()
        for t in USAGE_TYPES:
            self.usagecombo.append_text(t[0])
        self.usagecombo.set_active(0)
        self.usagecombo.connect("changed", on_changed)
        grid.attach(self.usagecombo, 0, 0, 1, 1)
        grid.show_all()
        VCardEntry.row += 6

    def add_entries(self, card):
        street = self.street.get_text()
        city = self.city.get_text()
        region = self.region.get_text()
        code = self.code.get_text()
        country = self.country.get_text()
        postbox = self.postbox.get_text()
        if street or city or region or code or country or postbox:
            card.add("adr")
            card.adr.value = vobject.vcard.Address(
                street, city,
                region, code,
                country, postbox)
            getattr(card,
                    self.entry_field).type_param = USAGE_TYPES[
                        self.usagecombo.get_active()][1]

        return card


class QRCodeVCardGtk(object):
    def __getitem__(self, itemname):
        return self.builder.get_object(itemname)

    def __init__(self, qr_code_update_func):
        self.qr_code_update_func = qr_code_update_func
        self.builder = Gtk.Builder()

        self.builder.add_from_file(
            get_data_file('ui', '%s.ui' % ('QrCodeVCard',)))
        self.builder.connect_signals(self)
        # TRANSLATORS: This refers to the most likely family name {0} -
        # given name {1} order in the language
        self.fullname_template = _("{1} {0}")
        self['entry1'].set_placeholder_text(
            _('[Required for a valid business card]'))
        self['entry2'].set_placeholder_text(
            _('[Required for a valid business card]'))
        for t in ENTRY_TYPES:
            self["comboboxtext1"].append_text(t[0])
        self["comboboxtext1"].set_active(0)
        self["entry1"].connect("icon-press", on_icon)
        self["entry2"].connect("icon-press", on_icon)
        VCardEntry.row = 3
        self.grid = self['qr_code_vcard']

    def create_vcard(self):
        self.j = vobject.vCard()
        self.j.add("n")
        f = self['entry2'].get_text()
        g = self['entry1'].get_text()
        self.j.n.value = vobject.vcard.Name(
            family=f, given=g)
        self.j.add("fn")
        self.j.fn.value = self.fullname_template.format(f, g)

        for e in VCardEntry.instances:
            self.j = e.add_entries(self.j)
        #print self.j.serialize()
        if f:
            self['image2'].set_from_icon_name(
                'object-select-symbolic', Gtk.IconSize.BUTTON)
        else:
            self['image2'].set_from_icon_name(
                'action-unavailable-symbolic', Gtk.IconSize.BUTTON)
        if g:
            self['image1'].set_from_icon_name(
                'object-select-symbolic', Gtk.IconSize.BUTTON)
        else:
            self['image1'].set_from_icon_name(
                'action-unavailable-symbolic', Gtk.IconSize.BUTTON)

    def on_activated(self):
        pass

    def on_prepared(self):
        pass

    def on_fullname_switch(self, widget, data=None):
        if self.fullname_template == "{0} {1}":
            self.fullname_template = "{1} {0}"
        else:
            self.fullname_template = "{0} {1}"
        self.on_changed(None)

    def on_changed(self, widget):
        if type(widget) == Gtk.Entry:
            show_clear_icon(widget)
        elif type(widget) == Gtk.ComboBoxText:
            try:
                show_clear_icon(widget.get_child())
            except:
                pass
        g = self['entry1'].get_text()
        f = self['entry2'].get_text()
        self['label4'].set_text(self.fullname_template.format(f, g))
        self.create_vcard()
        self.qr_code_update_func(self.j.serialize())
        return True

    def on_add_entry(self, widget):
        e_id = self['comboboxtext1'].get_active()
        entry_type = ENTRY_TYPES[e_id][1]
        #text = ENTRY_TYPES[e_id][0]
        if entry_type in ["email", "url"]:
            GenericWithUsage(e_id, self.builder.get_object,
                             "grid1").create_widget(self.on_changed)
        elif entry_type == "tel":
            Phone(e_id, self.builder.get_object,
                  "grid1").create_widget(self.on_changed)
        elif entry_type == "adr":
            Address(e_id, self.builder.get_object,
                    "grid1").create_widget(self.on_changed)
        elif entry_type in ["title", "org", "role", "nickname", "note"]:
            VCardEntry(e_id, self.builder.get_object,
                       "grid1").create_widget(self.on_changed)
        def update_scroll():
            self["scrolledwindow1"].get_vadjustment().set_value(
                self["scrolledwindow1"].get_vadjustment().get_upper())

        GObject.idle_add(update_scroll)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk  # pylint: disable=E0611
from . GtkHelpers import clear_text_entry, show_clear_icon
from qreator_lib.i18n import _
from qreator_lib.helpers import get_data_file


MESSAGE_TYPES = [
    # TRANSLATORS: this refers to a phone call QR code type
    (_("Call"), "tel"),
    # TRANSLATORS: this refers to an SMS message QR code type
    (_("Text Message"), "SMS"),
    ]


class QRCodeSMSGtk(object):
    def __init__(self, qr_code_update_func):
        self.qr_code_update_func = qr_code_update_func
        self.builder = Gtk.Builder()

        self.builder.add_from_file(
            get_data_file('ui', '%s.ui' % ('QrCodeSMS',)))
        self.grid = self.builder.get_object('qr_code_sms')

        # holds the SMS and Cell selector
        self.messagecombo = self.builder.get_object('comboboxType')
        for t in MESSAGE_TYPES:
            self.messagecombo.append_text(t[0])
        self.messagecombo.set_active(0)
        self.messagecombo.connect(
            "changed", self.on_comboboxType_changed, None)

        self.text = self.builder.get_object('scrolledwindow2')
        # tel is the default message type
        self.text.hide()
        self.builder.get_object('labelMessage').hide()
        self.textbuffer = self.builder.get_object(
            'textviewMessage').get_buffer()
        # TRANSLATORS: this refers to an SMS message
        self.placeholdermessage = _(
            "  [Text message. Some code readers might not support this QR code type]")  # pylint: disable=E0501
        self.textbuffer.set_text(self.placeholdermessage)
        self.messagechanged = self.textbuffer.connect(
            "changed", self.on_textviewMessage_changed, None)

        self.builder.get_object('textviewMessage').connect(
            "focus-in-event", self.on_textviewMessage_focusin, None)
        self.builder.get_object('textviewMessage').connect(
            "focus-out-event", self.on_textviewMessage_focusout, None)

        self.number = self.builder.get_object('entryNumber')
        self.numberchanged = self.number.connect(
            "changed", self.on_entryNumber_changed, None)
        self.number.connect("icon-press", self.on_entryNumber_icon_press)

    # pylint: disable=W0613
    def on_entryNumber_icon_press(self, widget, icon, mouse_button):
        clear_text_entry(widget, icon)

    def on_textviewMessage_focusin(self, widget, data, data2):
        """placeholder text remove"""

        if self.textbuffer.get_text(self.textbuffer.get_start_iter(),
                                    self.textbuffer.get_end_iter(),
                                    False) == self.placeholdermessage:
            self.textbuffer.handler_block(self.messagechanged)
            self.textbuffer.set_text("")
            self.textbuffer.handler_unblock(self.messagechanged)

    def on_textviewMessage_focusout(self, widget, data, data2):
        """placeholder text put back"""
        if self.textbuffer.get_text(self.textbuffer.get_start_iter(),
                                    self.textbuffer.get_end_iter(),
                                    False) == "":
            self.textbuffer.handler_block(self.messagechanged)
            self.textbuffer.set_text(self.placeholdermessage)
            self.textbuffer.handler_unblock(self.messagechanged)

    def on_activated(self):
        pass

    def on_prepared(self):
        pass

    def on_comboboxType_changed(self, widget, data=None):
        if not MESSAGE_TYPES[widget.get_active()][1] == "tel":
            self.text.show()
            self.builder.get_object('labelMessage').show()
        else:
            self.text.hide()
            self.builder.get_object('labelMessage').hide()

        # simply changing the type should not trigger an update on the
        # qrcode
        if not (self.builder.get_object('entryNumber').get_text() == "" and
                self.textbuffer.get_text(self.textbuffer.get_start_iter(),
                                         self.textbuffer.get_end_iter(),
                                         False) == self.placeholdermessage):
            self.on_textviewMessage_changed(self.textbuffer)

    def on_entryNumber_changed(self, widget, data=None):
        # will be used to validate the entered numbers and format them
        # should be replaced in the future with a libphonenumber
        # AsYouTypeFormatter
        show_clear_icon(widget)
        widget.handler_block(self.numberchanged)
        widget.set_text(
            "".join(c for c in widget.get_text() if c in "+0123456789"))
        widget.handler_unblock(self.numberchanged)
        self.on_textviewMessage_changed(self.textbuffer)

    def on_textviewMessage_changed(self, widget, data=None):
        # The SMS data format is not standardized.
        # Possible formats are:
        # sms:number
        # smsto:number
        # sms:number:message
        # smsto:number:message
        # sms:number?body=message

        # the same with capital SMS or SMSTO
        # => we will use SMSTO:number:message
        # based on the tests I did with other readers/scanners
        active_type = self.builder.get_object('comboboxType').get_active()
        ty = MESSAGE_TYPES[active_type][1]
        n = self.builder.get_object('entryNumber').get_text()
        te = widget.get_text(widget.get_start_iter(),
                             widget.get_end_iter(),
                             False)
        if ty == "tel" or te == "" or te == self.placeholdermessage:
            self.qr_code_update_func("{0}:{1}".format(ty, n))
        else:
            self.qr_code_update_func("{0}:{1}:{2}".format(ty, n, te))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gi
gi.require_version('GtkChamplain', '0.12')
from gi.repository import Gtk, GtkChamplain, Clutter, Champlain
from qreator_lib.helpers import get_data_file
gi.require_version('GtkClutter', '1.0')
from gi.repository import GtkClutter


class QRCodeLocationGtk(object):
    def __init__(self, qr_code_update_func):
        self.qr_code_update_func = qr_code_update_func
        self.builder = Gtk.Builder()

        self.builder.add_from_file(
            get_data_file('ui', '%s.ui' % ('QrCodeLocation',)))
        self.grid = self.builder.get_object('qr_code_location')

        GtkClutter.init([])
        map_widget = GtkChamplain.Embed()
        map_widget.set_hexpand(True)
        map_widget.set_vexpand(True)

        map_grid = self.grid

        #map_grid.set_size_request(-1, 350)
        map_grid.attach(map_widget, 0, 0, 1, 1)

        self.map_view = map_widget.get_view()
        self.map_view.set_reactive(True)
        map_widget.connect('button-release-event',
                           self.on_map_widget_button_press_event,
                           self.map_view)

        # Get the current location, center the map on it, and initialize
        # other map features
        self.get_current_location()
        self.map_view.set_zoom_level(1)
        self.map_view.set_kinetic_mode(True)

        scale = Champlain.Scale()
        scale.connect_view(self.map_view)
        self.map_view.bin_layout_add(scale, Clutter.BinAlignment.START,
                                     Clutter.BinAlignment.END)

        self.grid.show_all()

    def on_activated(self):
        pass

    def on_prepared(self):
        pass

    def on_map_widget_button_press_event(self, actor, event, view):
        x, y = event.get_coords()
        lon, lat = view.x_to_longitude(x), view.y_to_latitude(y)

        self.builder.get_object('lat_entry').set_text(str(lat))
        self.builder.get_object('lon_entry').set_text(str(lon))

        self.qr_code_update_func('geo:{0},{1}'.format(str(lat), str(lon)))

        return True

    def get_current_location(self):
        '''Gets the current location from geolocation via using Geoclue-2.0'''

        gi.require_version('Geoclue', '2.0')
        from gi.repository import Geoclue

        Geoclue.Simple.new('qreator',
                           Geoclue.AccuracyLevel.EXACT,
                           callback=on_current_location_ready,
                           user_data=self)
        return True


def on_current_location_ready(self, widget, data=None):
    location = self.get_location()
    latitude = location.get_property('latitude')
    longitude = location.get_property('longitude')
    if latitude != 0 and longitude != 0:
        data.builder.get_object('lat_entry').set_text(str(latitude))
        data.builder.get_object('lon_entry').set_text(str(longitude))
        data.map_view.center_on(latitude, longitude)
        data.map_view.set_zoom_level(15)
    return



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os
import tempfile
import atexit
import cairo
import math
from gi.repository import Gtk, Gdk, GdkPixbuf, GObject  # pylint: disable=E0611
import logging
logger = logging.getLogger('qreator')

from qreator_lib.i18n import _
from qreator_lib import Window
from qreator_lib.helpers import get_media_file

from .QRCode import QRCode
from .QRCode import QRCodeOutput

from qreator.qrcodes.QRCodeType import QRCodeType

COL_DESC = 0
COL_PIXBUF = 1
COL_ID = 2

PAGE_NEW = 0
#PAGE_HISTORY = 1
PAGE_ABOUT = 1
PAGE_QR = 2


# See qreator_lib.Window.py for more details about how this class works
class QreatorWindow(Window):   # pylint: disable=W0232
    __gtype_name__ = "QreatorWindow"

    def finish_initializing(self, builder):  # pylint: disable=E1002
        """Set up the main window"""
        super(QreatorWindow, self).finish_initializing(builder)

        # Code for other initialization actions should be added here.

        # Initialize the clipboard
        # pylint: disable=W0201
        self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)

        # Initialize the style for the main toolbar
        context = self.ui.toolbar1.get_style_context()
        context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)

        self.ui.toolbuttonNew.set_sensitive(False)
        self.ui.mnu_new.set_sensitive(False)

        # Initialize the Cairo surface that will contain the QR code
        self.surface = None

        # Hide the notebook tabs
        self.ui.notebook1.set_show_tabs(False)

        # Initialize about dialog
        self.init_about_dialog()

        # Create printing dialog
        self.reset_print_dialog()

        # Initialize the QR types icon view
        self.init_qr_types()

        # Load the background texture in the QR code page
        self.texture = cairo.ImageSurface.create_from_png(
            get_media_file("pattern.png"))

        # Time to wait in milliseconds before switching to the next view
        # after having clicked once on an iconview icon
        self.iconview_activation_delay = 200

        # Initialize color editing
        self.current_color_bg = Gdk.Color(65535, 65535, 65535)  # White
        self.previous_color_bg = Gdk.Color(65535, 65535, 65535)
        self.current_color_fg = Gdk.Color(0, 0, 0)  # Black
        self.previous_color_fg = Gdk.Color(0, 0, 0)

        # Add an initial text, so that there is an initial QR code
        self.qr_code_placeholder = 'http://launchpad.net/qreator'

        # Prepare the qrcode for drag n drop
        self.tempfilepath = None

        # pylint: disable=W0613
        def on_drag_data_get(widget, drag_context, selection_data,
                             info, time):
            if self.tempfilepath is None:
                # pylint: disable=W0612
                handle, self.tempfilepath = tempfile.mkstemp(
                    suffix=".png", prefix="qreator_")
                pixbuf = self.get_pixbuf_from_drawing_area()
                pixbuf.savev(self.tempfilepath, 'png', [], [])
            selection_data.set_uris(["file://{}".format(self.tempfilepath), ])
            return True

        def remove_tempfile(path):
            if os.path.exists(path):
                os.remove(path)
            return False

        def on_drag_end(widget, drag_context):
            atexit.register(remove_tempfile, self.tempfilepath)
            self.tempfilepath = None

        self.ui.qr_drawingarea.drag_source_set(Gdk.ModifierType.BUTTON1_MASK,
                                               [], Gdk.DragAction.COPY)
        self.ui.qr_drawingarea.drag_source_add_uri_targets()
        self.ui.qr_drawingarea.connect("drag_data_get", on_drag_data_get)
        self.ui.qr_drawingarea.connect("drag_end", on_drag_end)

    def init_qr_types(self):
        # Set up the QR types shown in the main icon view
        self.ui.qr_types_view.set_text_column(COL_DESC)
        self.ui.qr_types_view.set_pixbuf_column(COL_PIXBUF)

        # By importing the different qrcode dataformats in the __init__ file,
        # they have automatically registered themselves in the base class.
        self.qr_types = [d(self.update_qr_code) for d in QRCodeType.dataformats]
        # pylint: disable=W0201
        self.qr_types_store = Gtk.ListStore(str, GdkPixbuf.Pixbuf, int)
        # ^ desc, icon, id ^
        self.qr_types_store.set_sort_column_id(COL_DESC,
                                               Gtk.SortType.ASCENDING)
        self.ui.qr_types_view.set_model(self.qr_types_store)
        self.fill_qr_types_store()

        self.curr_height = 0
        self.curr_width = 0

        for qr_type in self.qr_types:
            # keep the window from growing with each qrtype
            qr_type.widget.grid.hide()
            self.ui.qr_input_box.add(qr_type.widget.grid)
            qr_type.widget.on_prepared()

    def fill_qr_types_store(self):
        self.qr_types_store.clear()

        for qr_id, qr_type in enumerate(self.qr_types):
            icon = GdkPixbuf.Pixbuf.new_from_file_at_size(qr_type.icon_path, 64, 64)
            self.qr_types_store.append([qr_type.description,
                                        icon,
                                        qr_id])

    def on_qreator_window_check_resize(self, widget):
        '''We need this function to fix the issue described at
        http://bit.ly/LW94BO whereby the number of columns of the icon view
        widget stays fixed at the number set for the initial width of the
        window'''
        # Get the new size
        new_width = widget.get_size()[0]
        new_height = widget.get_size()[1]
        # If the size has changed...
        if(new_width != self.curr_width or new_height != self.curr_height):
            # Remember new size
            self.curr_width = new_width  # pylint: disable=W0201
            self.curr_height = new_height  # pylint: disable=W0201
            # and refill iconviews with icons to adjust columns number
            self.fill_qr_types_store()

    def init_about_dialog(self):
        # Initialize about dialog
        about = Gtk.AboutDialog()
        about.set_program_name("Qreator")
        about.set_copyright(
            "Copyright (c) 2012-2013 David Planella")
        about.set_website("http://davidplanella.org/project-showcase/qreator/")

        #FIXME: the version should be picked up from setup.py
        about.set_version('16.06.2')
        about.set_authors([
            'David Planella <david.planella@ubuntu.com>',
            'Stefan Schwarzburg <stefan.schwarzburg@googlemail.com>',
            'Michael Hall <mhall119@ubuntu.com>',
            'Andrew Starr-Bochicchio <andrewsomething@ubuntu.com>',
            'Jef Spaleta <jspaleta@fedoraproject.org>',
            'Fong LH <fonglh@gmail.com>',
            'Ilya Volchanetskiy <volchanetskiy@ukr.net>',
            'Felix Eckhofer <felix@eckhofer.com>'
            ])
        about.set_license(_('Distributed under the GPL v3 license.') +
                '\nhttp://www.opensource.org/licenses/gpl-3.0.html')
        about.set_website(None)
        about.set_logo_icon_name(None)

        # Workaround for bug #1031657, remove David from translation credits
        # unless the locale is Catalan
        import locale
        # pylint: disable=W0612
        default_locale, encoding = locale.getdefaultlocale()
        # Gtk.AboutDialog does not recognize https and would not render the
        # markup otherwise
        translator_credits = _("translator-credits").replace('https', 'http')
        try:
            language_code = default_locale.split('_')[0]
        except AttributeError:
            language_code = default_locale
        if language_code != 'ca':
            translator_credits = translator_credits.replace(
                'David Planella' + ' ' + 'http://launchpad.net/~dpm',
                '').strip()
        about.set_translator_credits(translator_credits)

        box = self.ui.about_box
        about.vbox.reparent(box)

        # Get rid of the 'Close' button
        for button in about.action_area.get_children():
            if button.get_property('label') == 'gtk-close':
                button.destroy()

##########################################
    # pylint: disable=W0613
    def on_toolbuttonNew_clicked(self, widget, data=None):
        '''Shows the home page'''
        self.ui.qr_types_view.unselect_all()
        self.ui.toolbuttonNew.set_sensitive(False)
        self.ui.mnu_new.set_sensitive(False)
        self.ui.notebook1.set_current_page(PAGE_NEW)

    def on_toolbuttonHistory_clicked(self, widget, data=None):
        '''Shows the history page'''
        pass  # self.ui.notebook1.set_current_page(PAGE_HISTORY)

    # pylint: disable=W0613
    def on_toolbuttonAbout_clicked(self, widget, data=None):
        '''Shows the about page'''
        self.ui.notebook1.set_current_page(PAGE_ABOUT)
        self.ui.toolbuttonNew.set_sensitive(True)
        self.ui.mnu_new.set_sensitive(True)

##########################################
    def on_qr_types_view_selection_changed(self, widget):
        if len(widget.get_selected_items()) > 0:
            GObject.idle_add(self.on_qr_types_view_item_activated,
                             widget, widget.get_selected_items()[0])

    def on_qr_types_view_item_activated(self, widget, item):
        '''Loads the UI for the appropriate QR type'''

        model = widget.get_model()
        qr_id = model[item][COL_ID]
        self.switch_qrcode_view(qr_id)

    def switch_qrcode_view(self, qr_id):
        self.ui.toolbuttonNew.set_sensitive(True)
        self.ui.mnu_new.set_sensitive(True)

        self.qr_types[qr_id].widget.on_activated()

        def switch_callback(page):
            self.ui.notebook1.set_current_page(page)
            return False  # stop this callback from being called again
        GObject.timeout_add(self.iconview_activation_delay, switch_callback,
                            PAGE_QR)

        for child in self.ui.qr_input_box.get_children():
            child.hide()
        self.ui.qr_input_box.get_children()[qr_id].show()

##########################################

    def get_pixbuf_from_drawing_area(self):
        window = self.ui.qr_drawingarea.get_window()

        src_x, src_y = self.get_centered_coordinates(self.ui.qr_drawingarea,
                                                     self.surface)
        image_height = self.surface.get_height()
        image_width = self.surface.get_width()

        return Gdk.pixbuf_get_from_window(window, src_x, src_y,
                                          image_width, image_height)

##########################################
    def reset_print_dialog(self):
        # pylint: disable=W0201
        self.print_dialog = Gtk.PrintOperation()
        self.print_dialog.set_n_pages(1)
        # pylint: disable=W0201
        self.print_handler = self.builder.get_object(
            "toolbuttonPrint").connect(
            "clicked", self.on_toolbuttonPrint_clicked)

    # pylint: disable=W0613
    def on_toolbuttonPrint_clicked(self, widget, data=None):
        self.builder.get_object("toolbuttonPrint").disconnect(
            self.print_handler)
        self.print_dialog.connect(
            "draw_page", self.draw_page)
        result = self.print_dialog.run(
            Gtk.PrintOperationAction.PRINT_DIALOG, None)
        self.reset_print_dialog()

    # pylint: disable=W0613
    def draw_page(self, operation=None, context=None, page_nr=None):
        ctx = context.get_cairo_context()
        ctx.set_source_surface(self.surface)
        # Render the image
        ctx.paint()
        return

    def on_toolbuttonSave_clicked(self, widget, data=None):
        if not self.surface:
            return

        dialog = Gtk.FileChooserDialog(_("Please choose a file"), self,
            Gtk.FileChooserAction.SAVE,
            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
             Gtk.STOCK_SAVE, Gtk.ResponseType.OK))

        filter_png = Gtk.FileFilter()
        filter_png.set_name(_("PNG images"))
        filter_png.add_mime_type("image/png")
        dialog.add_filter(filter_png)

        response = dialog.run()

        if response == Gtk.ResponseType.OK:
            # We cannot write directly from the surface, as the
            # Surface.write_to_png() method writes the image in the original
            # size returned by qrencode (i.e. non-scaled), and the
            # SurfacePattern does not have any methods to write to disk.
            # So we read the contents from the Gtk.DrawingArea, put them into
            # a Gdk.Pixbuf and use its 'savev' method to write to disk.

            pixbuf = self.get_pixbuf_from_drawing_area()

            pixbuf.savev(dialog.get_filename(), 'png', [], [])

        dialog.destroy()

    def on_toolbuttonCopy_clicked(self, widget, data=None):
        if not self.surface:
            return

        pixbuf = self.get_pixbuf_from_drawing_area()
        self.clipboard.set_image(pixbuf)

    def on_toolbuttonEdit_clicked(self, widget, data=None):
        if widget.get_active():
            self.ui.toolbar2.show()
        else:
            self.ui.toolbar2.hide()

##########################################

    def set_selector_colors(self):
        '''Preloads the color selector with the current and previous
        QR code colors'''
        colorsel = self.ui.color_selection_dialog.get_color_selection()
        if self.ui.radiobuttonfg.get_active():
            colorsel.set_current_color(self.current_color_fg)
            colorsel.set_previous_color(self.previous_color_fg)
        elif self.ui.radiobuttonbg.get_active():
            colorsel.set_current_color(self.current_color_bg)
            colorsel.set_previous_color(self.previous_color_bg)

    def on_toolbuttonColorSelector_clicked(self, widget, data=None):
        '''Loads the color selector dialog and saves the chosen
        current and previous QR code colors'''
        colorsel = self.ui.color_selection_dialog.get_color_selection()

        self.set_selector_colors()

        colordiag = self.ui.color_selection_dialog
        response = colordiag.run()

        # It seems that when invoked from Glade, the color selector dialog
        # returns an integer instead of Gtk.ResponseType.OK, and we need to
        # set this integer explicitly in the .ui file. The 'OK' button
        # returns '1'.
        if response == 1:
            if self.ui.radiobuttonfg.get_active():
                self.current_color_fg = colorsel.get_current_color()
                self.previous_color_fg = colorsel.get_previous_color()
            elif self.ui.radiobuttonbg.get_active():
                self.current_color_bg = colorsel.get_current_color()
                self.previous_color_bg = colorsel.get_previous_color()

        self.update_qr_code(self.qr_code_placeholder)

        self.ui.color_selection_dialog.hide()

    def on_radiobuttonfg_toggled(self, widget, data=None):
        '''Loads the foreground color in the selector when the foreground
        radio button is toggled'''
        self.set_selector_colors()

    def on_radiobuttonbg_toggled(self, widget, data=None):
        '''Loads the foreground color in the selector when the background
        radio button is toggled'''
        self.set_selector_colors()

    def on_toolbuttonColorSwap_clicked(self, widget, data=None):
        """Swaps the QR code's foreground and background colors"""

        self.previous_color_bg = self.current_color_bg
        self.previous_color_fg = self.current_color_fg

        self.current_color_bg = self.current_color_fg
        self.current_color_fg = self.previous_color_bg

        self.update_qr_code(self.qr_code_placeholder)

    def on_toolbuttonColorReset_clicked(self, widget, data=None):
        '''Reset QR code to defaults. At this point only the colors
        can be edited, so effectively it resets the foreground and
        background colors to black and white, respectively'''
        # pylint: disable=W0201
        self.current_color_bg = Gdk.Color(65535, 65535, 65535)  # White
        self.previous_color_bg = Gdk.Color(65535, 65535, 65535)
        self.current_color_fg = Gdk.Color(0, 0, 0)  # Black
        self.previous_color_fg = Gdk.Color(0, 0, 0)

        self.update_qr_code(self.qr_code_placeholder)

##########################################

    def update_qr_code(self, text):
        # pylint: disable=W0201
        self.qr_code_placeholder = text
        self.ui.qr_drawingarea.queue_draw()

    def get_centered_coordinates(self, drawing_area, surface):

        drawing_area_height = drawing_area.get_allocated_height()
        drawing_area_width = drawing_area.get_allocated_width()
        image_height = surface.get_height()
        image_width = surface.get_width()

        return (drawing_area_width / 2 - image_width / 2,
                drawing_area_height / 2 - image_height / 2)

    def on_qr_drawingarea_draw(self, widget, ctx, data=None):
        text = self.qr_code_placeholder

        ## Fill the background

        # Create a rounded rectanble
        x = 0.0
        y = 0.0
        width = widget.get_allocated_width()
        height = widget.get_allocated_height()
        aspect = 1.0
        corner_radius = height / 50.0

        radius = corner_radius / aspect
        degrees = math.pi / 180.0

        ctx.new_sub_path()
        ctx.arc(x + width - radius, y + radius, radius, -90 * degrees,
                0 * degrees)
        ctx.arc(x + width - radius, y + height - radius, radius, 0 * degrees,
                90 * degrees)
        ctx.arc(x + radius, y + height - radius, radius, 90 * degrees,
                180 * degrees)
        ctx.arc(x + radius, y + radius, radius, 180 * degrees, 270 * degrees)
        ctx.close_path()

        # Fill the rounded rectangle with a linear gradient
        lg = cairo.LinearGradient(0.0, 0.0, 500.0, 500.0)
        lg.add_color_stop_rgba(0, 0.27, 0.27, 0.27, 1)  # 1,1,1 is white
        lg.add_color_stop_rgba(1, 0.22, 0.22, 0.22, 1)

        ctx.set_source(lg)
        ctx.fill()

        # Load a texture and overlay it to the gradient, tiled
        pattern = cairo.SurfacePattern(self.texture)
        pattern.set_extend(cairo.EXTEND_REPEAT)
        ctx.set_source(pattern)
        ctx.paint()

        if not text == '':
            ## Create the QR code
            qr_code = QRCode()  # output_size = int(min(width, height) * 0.9))

            # Note that we need to convert the 16-bit RGB values from
            # Gdk.Color to the 8-bit RGB values that PIL expects
            # pylint: disable=W0201
            self.surface = qr_code.encode(text,
                                current_color_fg=(
                                    self.current_color_fg.red / 256,
                                    self.current_color_fg.green / 256,
                                    self.current_color_fg.blue / 256),
                                previous_color_fg=(
                                    self.previous_color_fg.red / 256,
                                    self.previous_color_fg.green / 256,
                                    self.previous_color_fg.blue / 256),
                                current_color_bg=(
                                    self.current_color_bg.red / 256,
                                    self.current_color_bg.green / 256,
                                    self.current_color_bg.blue / 256),
                                previous_color_bg=(
                                    self.previous_color_bg.red / 256,
                                    self.previous_color_bg.green / 256,
                                    self.previous_color_bg.blue / 256),
                                output_type=QRCodeOutput.CAIRO_SURFACE)

            # Center the image in the drawing area
            centered_width, centered_height = \
                self.get_centered_coordinates(widget, self.surface)
            ctx.translate(centered_width, centered_height)

            # Set the surface as the context's source
            ctx.set_source_surface(self.surface)

            # Render the image
            ctx.paint()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

### DO NOT EDIT THIS FILE ###

"""Helpers for an Ubuntu application."""
import logging
import os
import xdg

from . qreatorconfig import get_data_file
from . Builder import Builder

from locale import gettext as _


class VendorError(Exception):
    """Raised by plugins if the required vendor infractucture is missing"""
    def __init__(self, value=None):
        if value is None:
            value = "Plugin disabled due to missing vendor infrastructure."
        self.value = value
    def __str__(self):
        return repr(self.value)


def get_builder(builder_file_name):
    """Return a fully-instantiated Gtk.Builder instance from specified ui
    file

    :param builder_file_name: The name of the builder file, without extension.
        Assumed to be in the 'ui' directory under the data path.
    """
    # Look for the ui file that describes the user interface.
    ui_filename = get_data_file('ui', '%s.ui' % (builder_file_name,))
    if not os.path.exists(ui_filename):
        ui_filename = None

    builder = Builder()
    builder.set_translation_domain('qreator')
    builder.add_from_file(ui_filename)
    return builder


# Owais Lone : To get quick access to icons and stuff.
def get_media_file(media_file_name):
    media_filename = get_data_file('media', '%s' % (media_file_name,))
    if not os.path.exists(media_filename):
        media_filename = None

    return media_filename


def get_cache_file(file_name):
    base = os.path.join(xdg.BaseDirectory.xdg_cache_home, 'qreator')
    path = os.path.join(base, file_name)
    if not os.path.exists(base):
        os.makedirs(base)
    return path


class NullHandler(logging.Handler):
    def emit(self, record):
        pass

def set_up_logging(opts):
    # add a handler to prevent basicConfig
    root = logging.getLogger()
    null_handler = NullHandler()
    root.addHandler(null_handler)

    formatter = logging.Formatter("%(levelname)s:%(name)s: %(funcName)s() '%(message)s'")

    logger = logging.getLogger('qreator')
    logger_sh = logging.StreamHandler()
    logger_sh.setFormatter(formatter)
    logger.addHandler(logger_sh)

    lib_logger = logging.getLogger('qreator_lib')
    lib_logger_sh = logging.StreamHandler()
    lib_logger_sh.setFormatter(formatter)
    lib_logger.addHandler(lib_logger_sh)

    plugin_logger = logging.getLogger('qrcodes')
    plugin_logger_sh = logging.StreamHandler()
    plugin_logger_sh.setFormatter(formatter)
    plugin_logger.addHandler(lib_logger_sh)

    # Set the logging level to show debug messages.
    if opts.verbose:
        logger.setLevel(logging.DEBUG)
        plugin_logger.setLevel(logging.DEBUG)
        logger.debug('logging enabled')
        if opts.verbose > 1:
            lib_logger.setLevel(logging.DEBUG)

def get_help_uri(page=None):
    # help_uri from source tree - default language
    here = os.path.dirname(__file__)
    help_uri = os.path.abspath(os.path.join(here, '..', 'help', 'C'))

    if not os.path.exists(help_uri):
        # installed so use gnome help tree - user's language
        help_uri = 'qreator'

    # unspecified page is the index.page
    if page is not None:
        help_uri = '%s#%s' % (help_uri, page)

    return help_uri

def show_uri(parent, link):
    from gi.repository import Gtk # pylint: disable=E0611
    screen = parent.get_screen()
    Gtk.show_uri(screen, link, Gtk.get_current_event_time())

def alias(alternative_function_name):
    '''see http://www.drdobbs.com/web-development/184406073#l9'''
    def decorator(function):
        '''attach alternative_function_name(s) to function'''
        if not hasattr(function, 'aliases'):
            function.aliases = []
        function.aliases.append(alternative_function_name)
        return function
    return decorator



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

### DO NOT EDIT THIS FILE ###

__all__ = [
    'project_path_not_found',
    'get_data_file',
    'get_data_path',
    ]

# Where your project will look for your data (for instance, images and ui
# files). By default, this is ../data, relative your trunk layout
__qreator_data_directory__ = '/usr/share/qreator/'
__license__ = 'GPL-3'
__version__ = '20.2.1'

import os

from locale import gettext as _

class project_path_not_found(Exception):
    """Raised when we can't find the project directory."""


def get_data_file(*path_segments):
    """Get the full path to a data file.

    Returns the path to a file underneath the data directory (as defined by
    `get_data_path`). Equivalent to os.path.join(get_data_path(),
    *path_segments).
    """
    return os.path.join(get_data_path(), *path_segments)


def get_data_path():
    """Retrieve qreator data path

    This path is by default <qreator_lib_path>/../data/ in trunk
    and /usr/share/qreator in an installed version but this path
    is specified at installation time.
    """

    # Get pathname absolute or relative.
    path = os.path.join(
        os.path.dirname(__file__), __qreator_data_directory__)

    abs_data_path = os.path.abspath(path)
    if not os.path.exists(abs_data_path):
        raise project_path_not_found

    return abs_data_path


def get_version():
    return __version__



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk  # pylint: disable=E0611


def clear_text_entry(widget, icon):
    if icon == Gtk.EntryIconPosition.SECONDARY:
        widget.set_text("")
        show_clear_icon(widget)
    elif icon == Gtk.EntryIconPosition.PRIMARY:
        widget.select_region(0, -1)
        widget.grab_focus()


def show_clear_icon(widget):
    """
    Show the clear icon whenever the field is not empty
    """
    if widget.get_text() != "":
        widget.set_icon_from_stock(Gtk.EntryIconPosition.SECONDARY,
            Gtk.STOCK_CLEAR)
        # Reverse the icon if we are using an RTL locale
        if widget.get_direction() == Gtk.TextDirection.RTL:
            pb = widget.get_icon_pixbuf(
                Gtk.EntryIconPosition.SECONDARY).flip(True)
            widget.set_icon_from_pixbuf(Gtk.EntryIconPosition.SECONDARY, pb)
    else:
        widget.set_icon_from_stock(Gtk.EntryIconPosition.SECONDARY, None)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk
from qreator_lib.helpers import get_data_file


class QRCodeTextGtk(object):
    def __init__(self, qr_code_update_func):
        self.qr_code_update_func = qr_code_update_func
        builder = Gtk.Builder()

        builder.add_from_file(
            get_data_file('ui', '%s.ui' % ('QrCodeText',)))
        self.grid = builder.get_object('qr_code_text')

        textbuffer = builder.get_object('entryText').get_buffer()
        textbuffer.connect("changed", self.on_entryText_changed, None)

    def on_activated(self):
        pass

    def on_prepared(self):
        pass

    def on_entryText_changed(self, widget, data=None):
        self.qr_code_update_func(widget.get_text(widget.get_start_iter(),
                                            widget.get_end_iter(),
                                            False))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from .GtkHelpers import clear_text_entry, show_clear_icon
from qreator_lib.i18n import _
from qreator_lib.helpers import get_data_file
import gi
gi.require_version('NM', '1.0')
from gi.repository import Gtk, NM, GLib, GdkPixbuf


def _escape(st):
        """Replace problematic characters with the escaped versions.

        Note that the backslash must be the first character to replace"""
        esc_string = st
        for c in ["\\", ",", ":", ";"]:
            esc_string = esc_string.replace(c, "\\" + c)
        return esc_string
    

class QRCodeWifiGtk(object):
    def __init__(self, qr_code_update_func):
        self.qr_code_update_func = qr_code_update_func
        self.builder = Gtk.Builder()

        self.builder.add_from_file(
            get_data_file('ui', '%s.ui' % ('QrCodeWifi',)))
        self.grid = self.builder.get_object('qr_code_wifi')
        self.builder.connect_signals(self)

        self.comboboxtextSecurity = self.builder.get_object(
            'comboboxtextSecurity')
        self.entryPassword = self.builder.get_object('entryPassword')

        # SSID
        self.entrySSID = self.builder.get_object('entrySSID')
        self.comboboxSSID = self.builder.get_object('comboboxSSID')
        self.SSIDmodel = Gtk.ListStore(str, GdkPixbuf.Pixbuf)
        self.comboboxSSID.set_model(self.SSIDmodel)

        # create the renderers for the strength
        # for some reason, the text renderer already works
        strengthcell = Gtk.CellRendererPixbuf()
        self.comboboxSSID.pack_start(strengthcell, False)
        self.comboboxSSID.add_attribute(strengthcell, 'pixbuf', 1)

        icontheme = Gtk.IconTheme.get_default()
        size = 24
        self.strength_pixbufs = []
        for strength in ["none", "weak", "ok", "good", "excellent"]:
            self.strength_pixbufs.append(icontheme.load_icon(
                "network-wireless-signal-{0}-symbolic".format(strength),
                size, Gtk.IconLookupFlags.USE_BUILTIN))

        # Initialize combo boxes (Glade seems not to do it for us)
        self.comboboxtextSecurity.set_active(0)

        # TRANSLATORS: 'expand' refers to expanding the drop-down menu
        # that shows autodetected wireless network identifiers
        self.entrySSID.set_placeholder_text(
            _('[Network identifier - expand for autodetection]'))
        self.entryPassword.set_placeholder_text(_('[Network password]'))

    def update_ssids(self):
        # check if there are already ssids in the combobox
        ssidlistmodel = self.comboboxSSID.get_model()
        ssidelements = []
        item = ssidlistmodel.get_iter_first()
        while not item is None:
            ssidelements.append(ssidlistmodel.get_value(item, 0))
            item = ssidlistmodel.iter_next(item)
        
        # get the new ssids from the network manager
        # and add them to combobox
        ssids = get_ssids()
        for ssid in ssids:
            if not ssid[1] in ssidelements:
                p = self.strength_pixbufs[min(99, ssid[0]) // 25]
                self.SSIDmodel.append([ssid[1], p])
                self.comboboxSSID.set_model(self.SSIDmodel)

        return True  # -> repeat the timeout calls

    def update_wifi_qr_code(self, security=None, ssid=None, password=None):
        if not security:
            security = self.comboboxtextSecurity.get_active_text()
        if not ssid:
            ssid = self.entrySSID.get_text()
        if not password:
            password = self.entryPassword.get_text()

        if not security or ssid == '' or password == '':
            return

        wifi_qr_code = 'WIFI:T:{0};S:{1};P:{2};;'.format(_escape(security), _escape(ssid),
                                                         _escape(password))
        self.qr_code_update_func(wifi_qr_code)

    def on_comboboxtextSecurity_changed(self, widget, data=None):
        self.update_wifi_qr_code(security=widget.get_active_text())

    def on_entrySSID_icon_press(self, widget, icon, mouse_button):
        clear_text_entry(widget, icon)

    def on_comboboxSSID_changed(self, widget, data=None):
        show_clear_icon(self.entrySSID)
        self.update_wifi_qr_code(ssid=widget.get_child().get_text())

    def on_entryPassword_icon_press(self, widget, icon, mouse_button):
        clear_text_entry(widget, icon)

    def on_entryPassword_changed(self, widget, data=None):
        show_clear_icon(widget)
        self.update_wifi_qr_code(password=widget.get_text())

    def on_checkbutton1_clicked(self, widget):
        if widget.get_active():
            self.entryPassword.set_visibility(True)
        else:
            self.entryPassword.set_visibility(False)

    def on_activated(self):
        pass

    def on_prepared(self):
        # check for new ssids every 5 seconds
        GLib.timeout_add_seconds(5, self.update_ssids)


def get_ssids():
    """Returns a list of ssids sorted by the signal strength."""
    nmc = NM.Client.new()
    devs = nmc.get_devices()

    ssids = []
    for dev in devs:
        if dev.get_device_type() == NM.DeviceType.WIFI:
            for ap in dev.get_access_points():
                ssid_gbytes = ap.get_ssid()

                # Unknown SSIDs are None
                if ssid_gbytes:
                    ssid = NM.utils_ssid_to_utf8(ssid_gbytes.get_data())
                    ssids.append((ap.get_strength(), ssid))
    ssids.sort()
    ssids.reverse()   # strongest has the highest number -> should be first
    return ssids



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os

TEXTDOMAIN = 'qreator'
LOCALEDIR = '/usr/share/locale/'

import locale
from locale import gettext as _

if os.path.abspath(__file__).startswith('/opt'):
    LOCALEDIR = '/opt/extras.ubuntu.com/qreator/share/locale/'

locale.bindtextdomain(TEXTDOMAIN, LOCALEDIR)
locale.bind_textdomain_codeset(TEXTDOMAIN, 'UTF-8')
locale.textdomain(TEXTDOMAIN)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk, GObject
from .GtkHelpers import clear_text_entry, show_clear_icon
from qreator_lib.helpers import get_data_file
from qreator_lib.i18n import _
import requests
import json


class IsdgShortener(object):
    # TRANSLATORS: this refers to the is.gd URL shortening service
    name = _("Isgd")

    def __init__(self, url=None):
        self.api_url = "http://is.gd/create.php"
        self.url = url

    def short_url(self):
        self.r = requests.get("{0}?format=simple&url={1}".format(self.api_url,
                                                                 self.url))
        if self.r.content.startswith(b"Error"):
            raise Exception(self.r.content)
        return self.r.content.decode("UTF-8")


class TinyUrlShortener(object):
    # TRANSLATORS: this refers to the tinyurl.com URL shortening service
    name = _("TinyUrl")

    def __init__(self, url=None):
        self.api_url = "http://tinyurl.com/api-create.php"
        self.url = url

    def short_url(self):
        self.r = requests.get("{0}?url={1}".format(self.api_url, self.url))
        return self.r.content.decode("UTF-8")


class BitlyShortener(object):
    # TRANSLATORS: this refers to the bit.ly URL shortening service
    name = _("Bitly")

    def __init__(self, url=None):
        self.api_url = 'http://api.bit.ly/v3/shorten'
        self.url = url
        self.data = {"login": "",  # TODO
                     "apiKey": "",  # TODO
                     "longUrl": self.url}

    def short_url(self):
        self.r = requests.get(self.api_url, params=self.data)
        self.results = json.loads(self.r.content)
        return self.results["data"]["url"]


class GoogleShortener(object):
    # TRANSLATORS: this refers to the goo.gl URL shortening service
    name = _("Google")

    def __init__(self, url=None):
        self.api_key = "AIzaSyCU_pFNpACW4luWEZRgNnfWMEUdlnZyYQI"
        self.api_url = 'https://www.googleapis.com/urlshortener/v1/url'
        self.url = url
        self.params = {"key": self.api_key,
                       'Content-Type': 'application/json'}
        self.data = {"longUrl": self.url}

    def short_url(self):
        self.r = requests.post(self.api_url, headers=self.params,
                               data=json.dumps(self.data))
        self.results = json.loads(self.r.content)
        return self.results["id"]


SHORTENER_TYPES = [
    IsdgShortener,
    TinyUrlShortener,
    BitlyShortener,
    GoogleShortener,
    ]


class QRCodeURLGtk(object):
    def __init__(self, qr_code_update_func):
        self.qr_code_update_func = qr_code_update_func
        self.builder = Gtk.Builder()

        self.builder.add_from_file(
            get_data_file('ui', '%s.ui' % ('QrCodeURL',)))
        self.builder.connect_signals(self)
        self.grid = self.builder.get_object('qr_code_url')

        self.messagedialog = self.builder.get_object('messagedialog1')

        self.entry = self.builder.get_object('entryURL')
        self.combobox = self.builder.get_object('comboboxtextProtocol')

        # Initialize placeholder text (we need to do that because due to
        # a Glade bug they are otherwise not marked as translatable)
        self.entry.set_placeholder_text(_('[URL]'))
        self.combobox.set_active(0)

        # Currently four shortener options are available: isdg, google,
        # tinyurl, bitly
        # We are choosing isdg at the moment. Later we will allow to define
        # this in preferences.
        self.Shortener = SHORTENER_TYPES[0]

        # TRANSLATORS: leave '{0}' as it is, it will be replaced by the
        # chosen URL shortening service
        self.builder.get_object("togglebuttonShorten").set_tooltip_text(
            "Use the {0} online service to generate a short URL.".format(
                self.Shortener.name))
        self.builder.get_object("togglebuttonShorten").set_has_tooltip(True)

        self.entryhandler = self.entry.connect("changed",
                                               self.on_entryURL_changed)
        # Moved this signal from the glade file to here, to be able to block
        # it later
        self.iconhandler = self.entry.connect("icon-press",
                                              self.on_entryURL_icon_press)
        self.combobox.connect("changed", self.on_comboboxtextProtocol_changed)

    def on_togglebuttonShorten_toggled(self, widget):
        if widget.get_active():
            # not sensitive while shortener is making the network requests
            # not editable while the shortener button is toggled
            self.entry.set_sensitive(False)
            self.entry.set_editable(False)
            self.entry.handler_block(self.iconhandler)
            self.combobox.set_sensitive(False)
            self.long_address = self.address
            self.entry.set_icon_activatable(Gtk.EntryIconPosition.SECONDARY,
                                            False)
            # If the request takes longer, the interface should in the
            # meantime change to unsensitive button and entry, to show at
            # least some behaviour
            GObject.idle_add(self._shorten_and_display)
        else:
            self.entry.set_has_tooltip(False)
            self.entry.set_text(self.long_address)
            self.entry.set_editable(True)
            self.entry.handler_unblock(self.iconhandler)
            self.combobox.set_sensitive(True)
            self.qr_code_update_func(self.address)
            self.entry.set_icon_activatable(Gtk.EntryIconPosition.SECONDARY,
                                            True)

    def on_messagedialog1_response(self, widget, data=None):
        widget.hide()

    def _reset_shortener(self):
        self.builder.get_object('togglebuttonShorten').set_active(False)

    def _shorten_and_display(self):
        self.entry.set_sensitive(True)
        s = self.Shortener(url=self.address)
        try:
            self.short_address = s.short_url()
        except requests.exceptions.ConnectionError as e:
            # TRANSLATORS: leave '{0}' as it is, it will be replaced by the
            # exception's error message
            self.messagedialog.set_markup(
                _("A network connection error occured: {0}".format(e)))
            self.messagedialog.show()
            GObject.idle_add(self._reset_shortener)
            return
        except Exception as e:
            # TRANSLATORS: leave '{0}' as it is, it will be replaced by the
            # exception's error message
            self.messagedialog.set_markup(
                _("An error occured while trying to shorten the URL: {0}".format(e)))   # pylint: disable=E0501
            self.messagedialog.show()
            GObject.idle_add(self._reset_shortener)
            return
        self.entry.set_tooltip_text(self.long_address)
        self.entry.set_has_tooltip(True)
        self.entry.set_text(self.short_address.decode('utf-8'))
        self.entry.select_region(0, len(self.entry.get_text()))
        self.qr_code_update_func(self.short_address)

    def on_activated(self):
        pass

    def on_prepared(self):
        pass

    def update_url_qr_code(self, protocol=None, www=None):
        if not protocol:
            protocol = self.combobox.get_active_text()
        if not www:
            www = self.entry.get_text()

        if not protocol or www == '':
            return

        self.address = protocol + www

        self.qr_code_update_func(self.address)
        return False

    def on_entryURL_icon_press(self, widget, icon, mouse_button):
        clear_text_entry(widget, icon)

    def check_and_remove_protocol(self, widget):
        """
        Creates a list of protocols (as given by the combobox).
        Checks if the beginning of the text entry is in that list. If so,
        remove the protocol and select the appropriate protocol in the
        combobox.
        """
        for n, protocol in enumerate(
            [mr[0] for mr in self.combobox.get_model()]):
            if widget.get_text().strip().startswith(protocol):
                widget.set_text(widget.get_text().strip()[len(protocol):])
                self.combobox.set_active(n)
                return

    def on_entryURL_changed(self, widget, data=None):
        show_clear_icon(widget)
        self.check_and_remove_protocol(widget)
        self.update_url_qr_code()

    def on_comboboxtextProtocol_changed(self, widget, data=None):
        self.update_url_qr_code(protocol=widget.get_active_text())



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from qreator_lib.helpers import get_media_file


class DataformatsMount(type):
    """ http://martyalchin.com/2008/jan/10/simple-plugin-framework/"""
    def __init__(cls, name, bases, attrs):  # pylint: disable=W0231,W0613
        if not hasattr(cls, 'dataformats'):
            # This branch only executes when processing the mount
            # point itself.  So, since this is a new dataformat type,
            # not an implementation, this class shouldn't be
            # registered as a plugin. Instead, it sets up a list where
            # dataformats can be registered later.
            cls.dataformats = []
        else:
            # This must be a dataformat implementation, which should
            # be registered.  Simply appending it to the list is all
            # that's needed to keep track of it later.
            cls.dataformats.append(cls)


class QRCodeType(object, metaclass=DataformatsMount):
    """This class is the base for QRCode dataformats. It is not used itself.

    Classes implementing a dataformat type should provide the following class
    variables:
    ====================  =====================================================
    description           translateble, very short, used in the GUI
    cmdline_option        make this unique in Qreator, since it will be used to
    icon_name             the name of the icon in <qreator>/data/media/, which
                          should be used for this datatype
    ====================  =====================================================

    Classes implementing a dataformat type should also provide the following
    methods:
    ====================  =====================================================
    create_widget         This method is called during init to create the
                          widget for this datatype.
    create_code           The paramters are data format specific, but the
                          method should be able to recreate a code from saved
                          history data.
                          This method should return a single string to pass
                          to the general code update function.
    ====================  =====================================================
    """

    # replace the description, and make it translatable with _("")
    description = "DataFormat"
    icon_name = "dataformat.png"  # replace this in subclass
    cmdline_option = "--dataformat"  # replace this in subclass

    def __init__(self, qr_code_update_func):
        self.qr_code_update_func = qr_code_update_func
        self.icon_path = get_media_file(self.icon_name)
        self.create_widget()  # pylint: disable=E1101



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from qreator_lib.i18n import _
from .QRCodeType import QRCodeType
from .QRCodeWifiGtk import QRCodeWifiGtk

class QRCodeWifi(QRCodeType):
    description = _('Wifi network')
    icon_name = 'wifi.png'
    cmdline_option_long = "--wifi"
    cmdline_option_short = "-w"
    message = _("Create a QR code for WiFi settings")

    def create_widget(self):
        self.widget = QRCodeWifiGtk(self.qr_code_update_func)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os.path
import optparse

from qreator_lib.i18n import _
from gi.repository import Gtk, Gdk, GLib, GObject  # pylint: disable=E0611
from qreator import QreatorWindow
from qreator_lib import set_up_logging, get_version, get_data_path, VendorError
from qreator.qrcodes.QRCodeType import QRCodeType
import logging

# use dynamic importing to get all qrcode dataformats without
# knowing their names.
# That way, all the information of a qrcode dataformat can
# be kept in a single file, like a plugin.
import pkgutil
import qreator.qrcodes

plugin_warnings = []
for module_finder, qr, ispkg in pkgutil.iter_modules(qreator.qrcodes.__path__):
    name = "qreator.qrcodes." + qr
    try:
        __import__(name, fromlist=[])
    except VendorError as e:
        plugin_warnings.append("%s: %s." % (name, e))


def parse_options(qr_types):
    """Support for command line options"""
    parser = optparse.OptionParser(version="%%prog %s" % get_version())
    parser.add_option(
        "-v", "--verbose", action="count", dest="verbose",
        help=_("Show debug messages (-vv debugs qreator_lib also)"))

    for n, qr in enumerate(qr_types):
        parser.add_option(
            qr.cmdline_option_short, qr.cmdline_option_long,
            dest="view", action="store_const", const=n,
            help=qr.message)

    (options, args) = parser.parse_args()

    set_up_logging(options)
    return options


def main():
    'constructor for your class instances'
    # Fix icons which are missing in 12.04
    icontheme = Gtk.IconTheme.get_default()
    icontheme.append_search_path(os.path.join(get_data_path(), "media"))
    settings = Gtk.Settings.get_default()
    settings.set_property("gtk-application-prefer-dark-theme", True)

    # Run the application
    window = QreatorWindow.QreatorWindow()

    qr_types = [d(window.update_qr_code) for d in QRCodeType.dataformats]
    options = parse_options(qr_types)

    logger = logging.getLogger('qrcodes')
    for plugin_warning in plugin_warnings:
        logger.warn(plugin_warning)
    
    window.qr_types = qr_types
    window.show()
    if getattr(options, 'view', None) is not None:
        qr_id = options.view
        window.switch_qrcode_view(qr_id)
        window.ui.notebook1.set_current_page(QreatorWindow.PAGE_QR)
    GObject.threads_init()
    GLib.threads_init()
    Gdk.threads_init()
    Gtk.main()
